/****************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 13/01/2013

CopyRight 2013 all rights are reserved


********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author)
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable copyright laws.
All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws (Italian or International ones), as well as to
civil liability for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following
software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*/

#include "init.h"

extern void delay_ms (int);

//**************************************************
// Global variables
//**************************************************

unsigned char motor_modality = 0;


//it points to the stepper motor state maximum number depends on the stepper modality)
signed char current_state = 0;

// Depending on the stepper modality the number of states are different
unsigned char number_of_states = 0;

// It indicates the level of the emergency stop out. It should be changed with the functions set and reset
// 0: Deactivated  1: Activated
unsigned char emergency_stop_out = 0;

// It indicates the level of the emergency stop out. It should be changed with the functions set and reset
// 0: Deactivated  1: Activated
unsigned char emergency_stop_in = 0;


//************************************
// Stepper Motor Settings
//************************************

// Motor torque refers the Stepper motor (is proportional to the current)
//int motor_torque = 0;

// Stepper current can not be set beyond that value but it could be changed at lower values.
int motor_max_current = 0;


// Array that contains the value for the H bridge INx inputs
// bit0 = IN1_HB_1
// bit1 = IN2_HB_1
// bit2 = IN1_HB_2
// bit3 = IN2_HB_2
unsigned char stepper_state[8];

//************************************
//DC Motor Settings
//************************************


//int DC_motor_max_speed = 0;
int DC_motor_wanted_speed = 0;
//int DC_motor_actual_speed = 0;
//int DC_motor_feedback_speed = 0;


//**************************************************
//             board_initialization
//**************************************************
signed char board_initialization (void) {


//**************************************************
// PORT Initialization
//**************************************************
	// Analog Inputs
	LATA = 0x00; 
 	TRISA = 0xFF;

	// Switches and H bridge signals
 	LATB = 0x00; 
 	TRISB = 0xF0;

	// CNC signals
 	LATC = 0x00;    	 
 	TRISC = 0b10111000;
 	
	// LED and dip switches
 	LATD = 0x00;  
 	TRISD = 0b11110000;

	// Analog Inputs
 	LATE = 0x00;
 	TRISE = 0xFF;

//**************************************************
// H bridges Initialization
//**************************************************
	IN1_HB_1 = DEACTIVATED;
	IN2_HB_1 = DEACTIVATED;

	IN1_HB_2 = DEACTIVATED;
	IN2_HB_2 = DEACTIVATED;

	ENABLE_HB_1 = DEACTIVATED;
	ENABLE_HB_2 = DEACTIVATED;

//**************************************************
//ADC Initialization
//**************************************************

	ADCON0bits.ACONV = 0;
	ADCON0bits.ACSCH = 0;

	//Disable buffer
	ADCON1bits.FIFOEN = 0;

	//set the analog input
	ANSEL0 = 0b11110011;
	ANSEL1 = 0x01;

	// ADC timing settings
	ADCON2 = 0b01111110;

//**************************************************
// Turn On peripherals test
//**************************************************
        
        //Delay Library Initialization
        setQuartz (40);

	LED_1 = LED_ON;
	delay_ms (1000);
	LED_1 = LED_OFF;

	LED_2 = LED_ON;
	delay_ms (1000);
	LED_2 = LED_OFF;

	TEMP = ACTIVATED;
	delay_ms (2000);
	TEMP = DEACTIVATED;
	
	if (ADC_read_VIN_TEST () < VIN_MIN){
		ERROR_LED = LED_ON;
		set_emergency_stop_out ();
		return (ERROR_VIN_MIN);
	}

	if (ADC_read_VIN_TEST () > VIN_MAX){
		ERROR_LED = LED_ON;
		set_emergency_stop_out ();
		return (ERROR_VIN_MAX);
	}

	return (OPERATION_PROPERLY_EXECUTED);
}



//**************************************************************************************************
//                                  OTHER FUNCTIONS
//**************************************************************************************************

//**************************************************
//             system_check
//**************************************************

signed char system_check (void) {

	unsigned char temperature;

	if (ADC_read_VIN_TEST () < VIN_MIN){
            stop_motor_rotation (MOTOR_ALL);
            ERROR_LED = LED_ON;
            set_emergency_stop_out ();
            return (ERROR_VIN_MIN);
	}

	if (ADC_read_VIN_TEST () > VIN_MAX){
            stop_motor_rotation (MOTOR_ALL);
            ERROR_LED = LED_ON;
            set_emergency_stop_out ();
            return (ERROR_VIN_MAX);
	}
	
	temperature = ADC_read_TEMP ();	

	if (temperature > TEMP_MAX){ 
            turn_on_FAN ();
            stop_motor_rotation (MOTOR_ALL);
            ERROR_LED = LED_ON;
            set_emergency_stop_out ();
            while (1);
	}

	if (temperature > TEMP_WARNING_1){
            turn_on_FAN ();
	} else
        if (temperature < TEMP_WARNING_1 - TEMP_HYSTERESIS)
            turn_off_FAN ();

  
	if (ADC_read_motor_current (MOTOR_1) > motor_max_current){ 
            stop_motor_rotation (MOTOR_ALL);
            ERROR_LED = LED_ON;
            set_emergency_stop_out ();
            while (1);
	}

	if (ADC_read_motor_current (MOTOR_2) > motor_max_current){
            stop_motor_rotation (MOTOR_ALL);
            ERROR_LED = LED_ON;
            set_emergency_stop_out ();
            while (1);
	}

        return (0);
}


//**************************************************
//             initialize_DC_motor
//**************************************************
#ifdef DC_MOTOR
signed char initialize_DC_motor (unsigned char modality, int pwm_frequency, int max_current ) {
	
	motor_max_current = max_current;

	if (emergency_stops_check () == ERROR_EMG_STOP_ACTIVE)
            return (ERROR_EMG_STOP_ACTIVE);

	//*************************************************
	// SIGNED_MAGNITUDE Modality
	//*************************************************

	if (modality == SIGNED_MAGNITUDE) {

            // Open Timer 2 as PWM time base
            OpenTimer2 (TIMER_INT_OFF & T2_PS_1_16 & T2_POST_1_1);

            OpenPWM2(pwm_SIGNED_MAGNITUDE_frequency_table[pwm_frequency]);
            OpenPWM1(pwm_SIGNED_MAGNITUDE_frequency_table[pwm_frequency]);

            motor_modality = SIGNED_MAGNITUDE;
	}


	//*************************************************
	// LAP Modality
	//*************************************************
	if (modality == LAP) {

            // Complemtary Mode for PWM0-PWM1 and PWM2-PWM3
            PWMCON0 = 0b00110000;

            // Set prescaler to 4
            PTCON0 = 0b00000100;

            // Set PWM Frequency according to the PWM frequency table
            power_control_PWM_set_period (pwm_LAP_frequency_table[pwm_frequency]);

            // Set Duty Cycle at 50% keeping the motors active but a speed 0 rad/s
            set_duty_cycle_DC0 (POWER_CONTROL_HALF_RESOLUTION);
            set_duty_cycle_DC1 (POWER_CONTROL_HALF_RESOLUTION);

            // Start PWM
            PTCON1bits.PTEN = 0x01;

            //Enables the motors
            ENABLE_HB_1 = ACTIVATED;
            ENABLE_HB_2 = ACTIVATED;

            motor_modality = LAP;
	}
	return (OPERATION_PROPERLY_EXECUTED);
}

#endif

//**************************************************
//             power_control_PWM_set_period
//**************************************************
void power_control_PWM_set_period (int period) {

    PTPERL = (unsigned char) (period & 0x00FF);
    PTPERH = (unsigned char) ((period & 0xFF00) >> 8);
}


//**************************************************
//             set_duty_cycle_DC0
//**************************************************
void set_duty_cycle_DC0 (int duty_cycle) {

    ENABLE_HB_1 = ACTIVATED;
    PDC0L = (unsigned char) (duty_cycle & 0x00FF);
    PDC0H = (unsigned char) ((duty_cycle & 0xFF00) >> 8);
}


//**************************************************
//             set_duty_cycle_DC1
//**************************************************
void set_duty_cycle_DC1 (int duty_cycle) {

    ENABLE_HB_2 = ACTIVATED;
    PDC1L = (unsigned char) (duty_cycle & 0x00FF);
    PDC1H = (unsigned char) ((duty_cycle & 0xFF00) >> 8);
}

//**************************************************
//             rotate_motor_clockwise
//**************************************************
#ifdef DC_MOTOR
signed char rotate_motor_clockwise (unsigned char motor, int speed){

    if (emergency_stops_check () == ERROR_EMG_STOP_ACTIVE)
        return (ERROR_EMG_STOP_ACTIVE);

    // If the motor madality has been not set, the function returns an error
    if (motor_modality == 0)
        return (ERROR_MODALITY_NOT_DEFINED);

    // Speed check
    if (speed > MAX_SPEED)
        speed = MAX_SPEED;

    if (speed < MIN_SPEED)
        speed = MIN_SPEED;

    DC_motor_wanted_speed = speed;


    if (motor_modality == SIGNED_MAGNITUDE) {

        if (motor == MOTOR_1) {
            IN1_HB_1 = DEACTIVATED;
            IN2_HB_1 = ACTIVATED;
            SetDCPWM1 (DC_motor_wanted_speed);
            ENABLE_HB_1 = ACTIVATED;
        }

        if (motor == MOTOR_2) {
            IN1_HB_2 = DEACTIVATED;
            IN2_HB_2 = ACTIVATED;
            SetDCPWM2(DC_motor_wanted_speed);
            ENABLE_HB_2 = ACTIVATED;
        }

        if (motor == MOTOR_ALL) {
            IN1_HB_1 = DEACTIVATED;
            IN2_HB_1 = ACTIVATED;
            SetDCPWM1 (DC_motor_wanted_speed);
            ENABLE_HB_1 = ACTIVATED;

            IN1_HB_2 = DEACTIVATED;
            IN2_HB_2 = ACTIVATED;
            SetDCPWM2(DC_motor_wanted_speed);
            ENABLE_HB_2 = ACTIVATED;
        }
    }

    if (motor_modality == LAP ) {

        if (speed > POWER_CONTROL_HALF_RESOLUTION)
            speed = POWER_CONTROL_HALF_RESOLUTION;

        if ( motor == MOTOR_1) {
            set_duty_cycle_DC0 ( POWER_CONTROL_HALF_RESOLUTION + speed);
        }

        if ( motor == MOTOR_2) {
            set_duty_cycle_DC1 ( POWER_CONTROL_HALF_RESOLUTION + speed);
        }

        if ( motor == MOTOR_ALL) {
            set_duty_cycle_DC0 ( POWER_CONTROL_HALF_RESOLUTION + speed);
            set_duty_cycle_DC1 ( POWER_CONTROL_HALF_RESOLUTION + speed);
        }
    }
    return (OPERATION_PROPERLY_EXECUTED);
}
#endif


//**************************************************
//             rotate_motor_anticlockwise
//**************************************************
#ifdef DC_MOTOR
signed char rotate_motor_anticlockwise (unsigned char motor, int speed){

    if (emergency_stops_check () == ERROR_EMG_STOP_ACTIVE)
        return (ERROR_EMG_STOP_ACTIVE);

    // If the motor madality has been not set, the function returns an error
    if (motor_modality == 0)
        return (ERROR_MODALITY_NOT_DEFINED);

    // Speed check
    if (speed > MAX_SPEED)
        speed = MAX_SPEED;

    if (speed < MIN_SPEED)
        speed = MIN_SPEED;

    DC_motor_wanted_speed = speed;

    if (motor_modality == SIGNED_MAGNITUDE) {

        if (motor == MOTOR_1) {
            IN1_HB_1 = ACTIVATED;
            IN2_HB_1 = DEACTIVATED;
            SetDCPWM1 (DC_motor_wanted_speed);
            ENABLE_HB_1 = ACTIVATED;
        }

        if (motor == MOTOR_2) {
            IN1_HB_2 = ACTIVATED;
            IN2_HB_2 = DEACTIVATED;
            SetDCPWM2(DC_motor_wanted_speed);
            ENABLE_HB_2 = ACTIVATED;
        }

        if (motor == MOTOR_ALL) {
            IN1_HB_1 = ACTIVATED;
            IN2_HB_1 = DEACTIVATED;
            SetDCPWM1 (DC_motor_wanted_speed);
            ENABLE_HB_1 = ACTIVATED;

            IN1_HB_2 = ACTIVATED;
            IN2_HB_2 = DEACTIVATED;
            SetDCPWM2(DC_motor_wanted_speed);
            ENABLE_HB_2 = ACTIVATED;
        }
    }

    if (motor_modality == LAP ) {

    if (speed > POWER_CONTROL_HALF_RESOLUTION)
        speed = POWER_CONTROL_HALF_RESOLUTION;

        if ( motor == MOTOR_1) {
            set_duty_cycle_DC0 ( POWER_CONTROL_HALF_RESOLUTION - speed);
        }

        if ( motor == MOTOR_2) {
            set_duty_cycle_DC1 (POWER_CONTROL_HALF_RESOLUTION - speed);
        }

        if ( motor == MOTOR_ALL) {
            set_duty_cycle_DC0 ( POWER_CONTROL_HALF_RESOLUTION - speed);
            set_duty_cycle_DC1 (POWER_CONTROL_HALF_RESOLUTION - speed);
        }
    }

    return (OPERATION_PROPERLY_EXECUTED);
}

#endif

//**************************************************
//             stop_motor_rotation
//**************************************************
signed char stop_motor_rotation (unsigned char motor){

    #ifdef DC_MOTOR
    if (motor_modality == SIGNED_MAGNITUDE) {

        // Close PWM Modules
        ClosePWM1 ();
        ClosePWM2 ();

        if (motor == MOTOR_1) {
            IN1_HB_1 = DEACTIVATED;
            IN2_HB_1 = DEACTIVATED;
        }

        if (motor == MOTOR_2) {
            IN1_HB_2 = DEACTIVATED;
            IN2_HB_2 = DEACTIVATED;
        }

        if (motor == MOTOR_ALL) {
            IN1_HB_1 = DEACTIVATED;
            IN2_HB_1 = DEACTIVATED;
            IN1_HB_2 = DEACTIVATED;
            IN2_HB_2 = DEACTIVATED;
        }
    }

     if (motor_modality == LAP) {

        // Close PWM Modules
        ClosePWM1 ();
        ClosePWM2 ();

        if (motor == MOTOR_1) {
            IN1_HB_1 = DEACTIVATED;
            IN2_HB_1 = DEACTIVATED;
            ENABLE_HB_1 = DEACTIVATED;
        }

        if (motor == MOTOR_2) {
            IN1_HB_2 = DEACTIVATED;
            IN2_HB_2 = DEACTIVATED;
            ENABLE_HB_2 = DEACTIVATED;
        }

        if (motor == MOTOR_ALL) {
            IN1_HB_1 = DEACTIVATED;
            IN2_HB_1 = DEACTIVATED;
            ENABLE_HB_1 = DEACTIVATED;
            IN1_HB_2 = DEACTIVATED;
            IN2_HB_2 = DEACTIVATED;
            ENABLE_HB_2 = DEACTIVATED;
        }
    }

    #endif

     if (motor_modality == STEPPER){

        // Close PWM Modules
        SetDCPWM2 (0);
        SetDCPWM1 (0);

        IN1_HB_1 = DEACTIVATED;
        IN2_HB_1 = DEACTIVATED;
        ENABLE_HB_1 = DEACTIVATED;

        IN1_HB_2 = DEACTIVATED;
        IN2_HB_2 = DEACTIVATED;
        ENABLE_HB_2 = DEACTIVATED;
    }

    return (OPERATION_PROPERLY_EXECUTED);
}


//**************************************************
//             initialize_RS485
//**************************************************
signed char initialize_RS485 (unsigned char baud_rate, unsigned char rx_interrupt_enable, unsigned char tx_interrupt_enable) {

//**************************************************
// RS485 Initialization
//**************************************************

    // Crystal 10MHz and 4x PLL (40MHz Clock)
    // Data Format 8 bit
    // 1 stop bit
    // Baud rate 19200bit/s
    // Interrupt off
    OpenUSART( tx_interrupt_enable &
    rx_interrupt_enable &
    USART_ASYNCH_MODE &
    USART_EIGHT_BIT &
    USART_CONT_RX &
    USART_BRGH_HIGH,
    baud_rate);

    // Turn On receiver
    RCSTAbits.CREN = 0x01;

    // Enable Interrupt
    PIE1bits.RCIE = 0x01;

    IPR1bits.RCIP = LOW_LEVEL_PRIORITY;

    // RS485 is in listening mode
    RS485_MODE = READ_ONLY;

    return (0);
}


//**************************************************
//             read_data_RS485
//**************************************************
signed char read_data_RS485 (unsigned char * data) {

    if (PIR1bits.RCIF == ACTIVATED)
            *data = RCREG;
    else
            return (ERROR_RX_DATA_NOT_READY);

    return (OPERATION_PROPERLY_EXECUTED);
}

//**************************************************
//             write_const_string_RS485
//**************************************************
signed char write_const_string_RS485 (const char *data) {

    RS485_MODE = READ_WRITE;

    putrsUSART (data);
    // wait until data is sent out
    while (BusyUSART());
    RS485_MODE = READ_ONLY;

    return (OPERATION_PROPERLY_EXECUTED);
}

//**************************************************
//            send_return_RS485
//**************************************************
void send_return_RS485 (void) {

    RS485_MODE = READ_WRITE;
    // carriage return
    WriteUSART(13);
    // wait until data is sent out
    while (BusyUSART());
    RS485_MODE = READ_ONLY;
}

//**************************************************
//             send_delete_RS485
//**************************************************
void send_delete_RS485 (void) {

    RS485_MODE = READ_WRITE;
    // carriage return
    WriteUSART(24);
    // wait until data is sent out
    while (BusyUSART());
    RS485_MODE = READ_ONLY;
}


//**************************************************
//             write_const_string_RS485
//**************************************************
signed char write_int_ASCII_RS485 (int value, unsigned char number_of_digits) {
	
    // The array size is 5 plus end of string \0
    unsigned char convertedInt [6];

    // Index used to shift to the right the digit
    unsigned char index = 0;

    // Integer is converted to string
    itoa ((unsigned char*) convertedInt, value,10);

    if (number_of_digits >0 ) {

        convertedInt[number_of_digits] = '\0';

        // Shift the digit to the right removing the empty one
        while (!(convertedInt[number_of_digits-1] <= '9' && convertedInt[number_of_digits-1] >= '0')){

            for (index = number_of_digits-1; index > 0; index--){
                convertedInt[index] = convertedInt[index-1];
                convertedInt[index-1] = ' ';
            }
        }
    }

    RS485_MODE = READ_WRITE;

    putsUSART (convertedInt);

    // wait until data is sent out
    while (BusyUSART());
//	// carriage return
//	WriteUSART(13);
//	// wait until data is sent out
//	while (BusyUSART());

    RS485_MODE = READ_ONLY;

    return (OPERATION_PROPERLY_EXECUTED);
}

//**************************************************
//             get_dip_switch_value
//**************************************************

unsigned char get_dip_switch_value (void) {

    unsigned char dip_switch = 0;


    if (DIP_SW_1 == GND)
            dip_switch |= 0x01;

    if (DIP_SW_2 == GND)
            dip_switch |= 0x02;

    if (DIP_SW_3 == GND)
            dip_switch |= 0x04;

    if (DIP_SW_4 == GND)
            dip_switch |= 0x08;

    return (dip_switch);
}

//**************************************************
//             ADC_read_buffer
//**************************************************
int ADC_read_buffer (void) {

    static int ADC_buffer_reading;

    // Activate ADC
    ADCON0bits.ADON = 1;
    ADCON0bits.GO_DONE = 1;

    while (ADCON0bits.GO_DONE);

    ADC_buffer_reading = (int) ADRESH;
    ADC_buffer_reading = ADC_buffer_reading << 2;
    ADC_buffer_reading = ADC_buffer_reading + (int) (ADRESL >> 6);

    return (ADC_buffer_reading);

} 

//**************************************************
//             ADC_read_POT_1
//**************************************************
int ADC_read_POT_1 (void ){
	
    // Select Group C Select
    ADCON0bits.ACMOD0 = 0;
    ADCON0bits.ACMOD1 = 1;

    //Select AN6 Channel within the group C
    ADCHS =  0b00000100;

    return (ADC_read_buffer ());
}

//**************************************************
//             ADC_read_POT_2
//**************************************************
int ADC_read_POT_2 (void ){
	
    // Select Group D
    ADCON0bits.ACMOD0 = 1;
    ADCON0bits.ACMOD1 = 1;

    //Select AN7 Channel within the group D
    ADCHS =  0b01000000;

    return (ADC_read_buffer ());
}

//**************************************************
//             ADC_read_TEMP
//**************************************************
int ADC_read_TEMP (void ){
	
    // Select Group A
    ADCON0bits.ACMOD0 = 0;
    ADCON0bits.ACMOD1 = 0;

    //Select AN8 Channel within the group A
    ADCHS =  0b00000010;

    return (ADC_read_buffer () >> 1);
}

//**************************************************
//             ADC_read_VIN_TEST
//**************************************************
int ADC_read_VIN_TEST (void ){
	
    // Select Group B
    ADCON0bits.ACMOD0 = 1;
    ADCON0bits.ACMOD1 = 0;

    //Select AN5 Channel within the group B
    ADCHS =  0b00010000;

    //return the voltage in volts
    return (ADC_read_buffer ()/ 23);
}


//**************************************************
//             ADC_read_motor_current
//**************************************************
int ADC_read_motor_current (unsigned char motor ) {


    switch (motor) {

        case MOTOR_1:
                        // Select Group A
                        ADCON0bits.ACMOD0 = 0;
                        ADCON0bits.ACMOD1 = 0;

                        //Select AN0 Channel within the group A
                        ADCHS =  0b00000000;
                        break;
        case MOTOR_2:
                        // Select Group B
                        ADCON0bits.ACMOD0 = 1;
                        ADCON0bits.ACMOD1 = 0;

                        //Select AN1 Channel within the group B
                        ADCHS =  0b00000000;
                        break;
        default:
                        break;
     }

    //return the current in mA
    return (ADC_read_buffer () * 10);
}


//**************************************************
//             initialize_stepper_motor
//**************************************************
signed char initialize_stepper_motor (unsigned char modality, unsigned char pwm_frequency, int max_current ) {

    if (emergency_stops_check () == ERROR_EMG_STOP_ACTIVE)
        return (ERROR_EMG_STOP_ACTIVE);

    IN1_HB_1 = DEACTIVATED;
    IN2_HB_1 = DEACTIVATED;

    IN1_HB_2 = DEACTIVATED;
    IN2_HB_2 = DEACTIVATED;

    ENABLE_HB_1 = ACTIVATED;
    ENABLE_HB_2 = ACTIVATED;


    // Open Timer 2 as PWM time base
    OpenTimer2 (TIMER_INT_OFF & T2_PS_1_16 & T2_POST_1_1);

    // Set the PWM frequency
    OpenPWM2(pwm_SIGNED_MAGNITUDE_frequency_table[pwm_frequency]);
    OpenPWM1(pwm_SIGNED_MAGNITUDE_frequency_table[pwm_frequency]);

    motor_max_current = max_current;

    SetDCPWM2 (0);
    SetDCPWM1 (0);

    switch (modality) {

                case FULL_STEP:
                                        stepper_state[0] = FULL_STEP_STATE_1;
                                        stepper_state[1] = FULL_STEP_STATE_2;
                                        stepper_state[2] = FULL_STEP_STATE_3;
                                        stepper_state[3] = FULL_STEP_STATE_4;
                                        number_of_states = FULL_STEP_NR_STATES;
                                        break;
                case FULL_STEP_2_PHASES:
                                        stepper_state[0] = FULL_STEP_2_PHASES_STATE_1;
                                        stepper_state[1] = FULL_STEP_2_PHASES_STATE_2;
                                        stepper_state[2] = FULL_STEP_2_PHASES_STATE_3;
                                        stepper_state[3] = FULL_STEP_2_PHASES_STATE_4;
                                        number_of_states = FULL_STEP_2_PHASES_NR_STATES;
                                        break;
                case HALF_STEP:
                                        stepper_state[0] = HALF_STEP_STATE_1;
                                        stepper_state[1] = HALF_STEP_STATE_2;
                                        stepper_state[2] = HALF_STEP_STATE_3;
                                        stepper_state[3] = HALF_STEP_STATE_4;
                                        stepper_state[4] = HALF_STEP_STATE_5;
                                        stepper_state[5] = HALF_STEP_STATE_6;
                                        stepper_state[6] = HALF_STEP_STATE_7;
                                        stepper_state[7] = HALF_STEP_STATE_8;
                                        number_of_states = HALF_STEP_NR_STATES;
                                        break;
                default:
                                        break;
             }

    motor_modality = STEPPER;

    return (OPERATION_PROPERLY_EXECUTED);
}


//**************************************************
//             step_motor_clockwise
//**************************************************
signed char step_motor_clockwise (int current) {
	
	if (emergency_stops_check () == ERROR_EMG_STOP_ACTIVE)
		return (ERROR_EMG_STOP_ACTIVE);

        #ifndef INTERRUPT_CONTROL
            SetDCPWM2 (current);
            SetDCPWM1 (current);
        #endif

	IN1_HB_1 = stepper_state[current_state] & 0x01;
	IN2_HB_1 = (stepper_state[current_state] & 0x02) >> 1;
	IN1_HB_2 = (stepper_state[current_state] & 0x04) >> 2;
	IN2_HB_2 = (stepper_state[current_state] & 0x08) >> 3;
	
	current_state++;

	if (current_state == number_of_states) {
            current_state = 0;
	}

	return (OPERATION_PROPERLY_EXECUTED);	
}


//**************************************************
//             step_motor_anticlockwise
//**************************************************
signed char step_motor_anticlockwise (int current) {

    if (emergency_stops_check () == ERROR_EMG_STOP_ACTIVE)
        return (ERROR_EMG_STOP_ACTIVE);

    #ifndef INTERRUPT_CONTROL
        SetDCPWM2 (current);
        SetDCPWM1 (current);
    #endif

    current_state--;

    if (current_state < 0) {
        current_state = number_of_states -1;
    }

    IN1_HB_1 = stepper_state[current_state] & 0x01;
    IN2_HB_1 = (stepper_state[current_state] & 0x02) >> 1;
    IN1_HB_2 = (stepper_state[current_state] & 0x04) >> 2;
    IN2_HB_2 = (stepper_state[current_state] & 0x08) >> 3;

    return (OPERATION_PROPERLY_EXECUTED);
}


//************************************************************
//             turn_on_FAN implementation
//************************************************************
void turn_on_FAN (void) {
	TEMP = ACTIVATED;
}

//************************************************************
//             turn_off_FAN implementation
//************************************************************
void turn_off_FAN (void) {
	TEMP = DEACTIVATED;
}

//************************************************************
//             is_emergency_stop_out_active
//************************************************************
unsigned char is_emergency_stop_out_active (void){

    if (emergency_stop_out == ACTIVATED)
            return (ACTIVATED);
    else
            return (DEACTIVATED);
}


//************************************************************
//             is_emergency_stop_in_active
//************************************************************
unsigned char is_emergency_stop_in_active (void){

    if (emergency_stop_in == ACTIVATED)
        return (ACTIVATED);
    else
        return (DEACTIVATED);
}

//************************************************************
//             set_emergency_stop_out
//************************************************************
unsigned char set_emergency_stop_out (void){
	
    EMG_STOP_OUT = ACTIVATED;
    emergency_stop_out = ACTIVATED;
    return (OPERATION_PROPERLY_EXECUTED);
}


//************************************************************
//             set_emergency_stop_in
//************************************************************
unsigned char set_emergency_stop_in (void){
	
    set_emergency_stop_out ();
    emergency_stop_out = ACTIVATED;
    emergency_stop_in = ACTIVATED;
    return (OPERATION_PROPERLY_EXECUTED);
}

//************************************************************
//             reset_emergency_stop_out
//************************************************************
unsigned char reset_emergency_stop_out (void){

    EMG_STOP_OUT = DEACTIVATED;
    emergency_stop_out = DEACTIVATED;
    return (OPERATION_PROPERLY_EXECUTED);
}


//************************************************************
//             reset_emergency_stop_out
//************************************************************
unsigned char reset_emergency_stop_in (void){

    reset_emergency_stop_out ();
    emergency_stop_out = DEACTIVATED;
    emergency_stop_in = DEACTIVATED;
    return (OPERATION_PROPERLY_EXECUTED);
}


//************************************************************
//             emergency_check
//************************************************************
unsigned char emergency_stops_check (void){

    if (is_emergency_stop_out_active ()) {
        stop_motor_rotation (MOTOR_ALL);
        return (ERROR_EMG_STOP_ACTIVE);
    }

    if (is_emergency_stop_in_active ()){
        stop_motor_rotation (MOTOR_ALL);
        return (ERROR_EMG_STOP_ACTIVE);
    }

    return (EMG_STOP_NOT_ACTIVATED);
}


//************************************************************
//             initialize_TIMER0
//************************************************************
void initialize_TIMER0 (unsigned char interrupt_priority) {

    // 16 bit Modality
    T0CONbits.T016BIT = 0;
    // Intenal Clock
    T0CONbits.T0CS = 0;

    // Enable Prescaler
    T0CONbits.PSA = 0;

    // Prescaler 4
    T0CONbits.T0PS0 = 1;
    T0CONbits.T0PS1 = 0;
    T0CONbits.T0PS2 = 0;

    // Enable Interrupt Timer0
    INTCONbits.TMR0IE = 1;

    if (interrupt_priority == LOW_LEVEL_PRIORITY) {
        // Enable Low Priority Interrupt
        INTCON2bits.TMR0IP = 0;
    } else {
        // Enable High Priority Interrupt
        INTCON2bits.TMR0IP = 1;
    }

    // Start Timer0
    T0CONbits.TMR0ON = 1;
}


//************************************************************
//             activate_signal_interupt
//************************************************************
void activate_signal_interupt (unsigned char signal, unsigned char interrupt_priority ) {

    if (signal == STEP_SIGNAL) {
        // Enable Interrupt for INT0 = STEP
        // INT0 is only High only High Priority
        INTCONbits.INT0IE = 0x01;
        INTCON2bits.INTEDG0 = RISING_EDGE;

    }

    if (signal == DIRECTION_SIGNAL) {
        // Enable Interrupt for INT1 = DIRECTION

        if (interrupt_priority == HIGH_LEVEL_PRIORITY) {
            INTCON3bits.INT1IE = ACTIVATED;
            INTCON3bits.INT1IP = HIGH_LEVEL_PRIORITY;
            INTCON2bits.INTEDG1 = RISING_EDGE;
        } else {
            INTCON3bits.INT1IE = ACTIVATED;
            INTCON3bits.INT1IP = LOW_LEVEL_PRIORITY;
            INTCON2bits.INTEDG1 = RISING_EDGE;
        }
        
    }

    if (signal == ENABLE_SIGNAL) {
        // Enable Interrupt for INT1 = ENABLE

        if (interrupt_priority == HIGH_LEVEL_PRIORITY) {
            INTCON3bits.INT2IE = ACTIVATED;
            INTCON3bits.INT2IP = HIGH_LEVEL_PRIORITY;
            INTCON2bits.INTEDG2 = RISING_EDGE;
        } else {
            INTCON3bits.INT2IE = ACTIVATED;
            INTCON3bits.INT2IP = LOW_LEVEL_PRIORITY;
            INTCON2bits.INTEDG2 = RISING_EDGE;
        }

    }

    if (signal == SWITCH_SIGNAL) {
        // Abilito le interruzioni su PORTB
        INTCONbits.RBIE = 1;
        // Abilito le interruzioni su PORTB come alta priorit
        INTCON2bits.RBIP = 1;
    }

}

//************************************************************
//             initialize_interrupt
//************************************************************
void initialize_interrupt (unsigned char interrupt_status) {

    if (interrupt_status == ACTIVATED) {
        // Enable two level priorities
        RCONbits.IPEN = 1;
        // Enable HIGH level Priprity
        INTCONbits.GIEH = 1;
        // Enable LOW level Priprity
        INTCONbits.GIEL = 1 ;

    } else {
        
        // Disable two level priorities
        RCONbits.IPEN = 0;
        // Disable HIGH level Priprity
        INTCONbits.GIEH = 0;
        // Disable LOW level Priprity
        INTCONbits.GIEL = 0 ;
    }

}


//************************************************************
//             set_error_status
//************************************************************
void set_error_status (unsigned char error) {
    error_status |= error;
}

//************************************************************
//             get_error_status
//************************************************************
unsigned char get_error_status (void){
    return (error_status) ;
}


//************************************************************
//             set_warning_status
//************************************************************
void set_warning_status (unsigned char warning){
    warning_status |= warning;
}

//************************************************************
//             get_warning_status
//************************************************************
unsigned char get_warning_status (void){
    return (warning_status);
}

