/****************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 13/01/2013
 
CopyRight 2013 all rights are reserved


********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) 
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).  
The software is owned by the Author, and is protected under applicable copyright laws. 
All rights are reserved. 
Any use in violation of the foregoing restrictions may subject the 
user to criminal sanctions under applicable laws (Italian or International ones), as well as to 
civil liability for the breach of the terms and conditions of this license. 
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following 
software doesn't aim to commercial use or monetary earning of any kind.    

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT, 
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*/

/****************************************************************************/

#include <xc.h>
#include <pwm.h>
#include <timers.h>
#include <usart.h>
#include <stdlib.h>
#include "delay.h"


#ifndef FLAG_INIT

#define FLAG_INIT

#define CLOCK_FREQUENCY 40

#define IN1_HB_1	LATBbits.LATB0
#define IN2_HB_1	LATBbits.LATB1

#define IN1_HB_2	LATBbits.LATB2
#define IN2_HB_2	LATBbits.LATB3

#define ENABLE_HB_1 	LATCbits.LATC2
#define ENABLE_HB_2 	LATCbits.LATC1

#define LED_1 		LATDbits.LATD0
#define LED_2 		LATDbits.LATD1
#define HEART_LED 	LATDbits.LATD0
#define ERROR_LED 	LATDbits.LATD1

#define RS485_OUT_ENABLE 	LATDbits.LATD2
#define RS485_MODE 		LATDbits.LATD2
#define READ_ONLY 0
#define READ_WRITE 1

#define TEMP LATDbits.LATD3

#define AUX_SW_1 PORTBbits.RB4
#define AUX_SW_2 PORTBbits.RB5

#define DIP_SW_1 PORTDbits.RD5
#define DIP_SW_2 PORTDbits.RD6
#define DIP_SW_3 PORTDbits.RD7
#define DIP_SW_4 PORTBbits.RB7

#define LED_ON 1
#define LED_OFF 0

#define VCC 1
#define GND 0

// General Purpose
#define ACTIVATED 1
#define DEACTIVATED 0

#define FLAG_SET 1
#define FLAG_CLEAR 0

#define LOW_LEVEL_PRIORITY 0
#define HIGH_LEVEL_PRIORITY 1

#define ENABLE_SIGNAL 0
#define STEP_SIGNAL 1
#define DIRECTION_SIGNAL 2
#define SWITCH_SIGNAL 3

#define RISING_EDGE 1
#define FALLING_EDGE 0

#define CNC_ENABLE 	PORTCbits.RC5
#define CNC_STEP 	PORTCbits.RC3
#define CNC_DIRECTION 	PORTCbits.RC4

#define CNC_STEP_FLAG   INTCONbits.INT0IF
#define CNC_DIRECTION_FLAG   INTCON3bits.INT1IF
#define CNC_ENABLE_FLAG INTCON3bits.INT2IF
#define CNC_SWITCH_FLAG INTCONbits.RBIF

#define EMG_STOP_OUT 	LATCbits.LATC0

#define EMG_STOP_FLTA 	PORTDbits.RD4
#define EMG_STOP_RB6 	PORTBbits.RB6

//System Error Status
#define ERROR_OVER_VOLTAGE 	1
#define ERROR_UNDER_VOLTAGE 	2
#define ERROR_OVER_CURRENT_M_1	3
#define ERROR_OVER_CURRENT_M_2  4
#define ERROR_OVER_TEMP 	5
#define ERROR_BOUDARY_SW_1  	6
#define ERROR_BOUDARY_SW_2 	7
#define ERROR_DELTA_VOLTAGE_O	8
#define ERROR_DELTA_VOLTAGE_B	9

volatile unsigned char error_status = 0;

//System Error Status
#define WARNING_MISSING_STEPS 	0b00000001    // 1
#define WARNING_DELTA_VOLTAGE_O 0b00000010    // 2
#define WARNING_DELTA_VOLTAGE_B 0b00000100    // 3
#define WARNING_TEMPERATURE     0b00001000    // 4
#define WARNING_COMMUNICATION 	0b00010000    // 5
#define WARNING_NOT_DEFINED_2 	0b00100000    // 6
#define WARNING_NOT_DEFINED_3 	0b01000000    // 7
#define WARNING_NOT_DEFINED_4 	0b10000000    // 8

volatile unsigned char warning_status = 0;

#define MOTOR_1 	1
#define MOTOR_2 	2
#define MOTOR_ALL 	3

// Stepper Motor madality
#define FULL_STEP 1
#define FULL_STEP_2_PHASES 2
#define HALF_STEP 3

// Motor madality
#define LAP 1
#define SIGNED_MAGNITUDE 2
#define STEPPER 3


// 10 bit PWM resolution
#define MAX_SPEED 1023
#define MIN_SPEED 0

//V input range
// Once VIN_MAX or VIN_MIN is reached, the system should be shouted down.
// At the system start Vin is measured. A delta afterword might mean a power supply
// Problem so a warnign is set.
#define VIN_MIN 11
#define VIN_MAX 28
#define VIN_DELTA_WARNING 2
#define VIN_DELTA_ERROR 3

//Temperature ranges
// Once TEMP_MAX is reached, the system should be shouted down.
// Reaching TEMP_WARNING_1 the fan should be turned ON.
// Reaching TEMP_WARNING_2 a warning signal is released
#define TEMP_MAX 65
#define TEMP_WARNING_1 35
#define TEMP_WARNING_2 45
#define TEMP_HYSTERESIS 6

//Current Max
//The actual current is MAX_CURRENT * 10 mA
#define MAX_PHASE_CURRENT_PEAK 500

//Stepper motor full step states
#define FULL_STEP_STATE_1 2
#define FULL_STEP_STATE_2 8
#define FULL_STEP_STATE_3 1
#define FULL_STEP_STATE_4 4
#define FULL_STEP_NR_STATES 4

//Stepper motor full step with two phases states
//Tourque is 1.4 higher than standard full step (current is two time higher)
#define FULL_STEP_2_PHASES_STATE_1 10
#define FULL_STEP_2_PHASES_STATE_2 9
#define FULL_STEP_2_PHASES_STATE_3 5
#define FULL_STEP_2_PHASES_STATE_4 6
#define FULL_STEP_2_PHASES_NR_STATES 4
 
//Stepper motor half step states
#define HALF_STEP_STATE_1 2
#define HALF_STEP_STATE_2 10
#define HALF_STEP_STATE_3 8
#define HALF_STEP_STATE_4 9
#define HALF_STEP_STATE_5 1
#define HALF_STEP_STATE_6 5
#define HALF_STEP_STATE_7 4
#define HALF_STEP_STATE_8 6
#define HALF_STEP_NR_STATES 8

//**************************************************
// ADC State Machine
//**************************************************

#define ADC_MOTOR_1_TEST 0
#define ADC_MOTOR_2_TEST 1
#define ADC_PHASE_1_TEST 0
#define ADC_PHASE_2_TEST 1
#define ADC_VIN_TEST 2
#define ADC_TEMP_TEST 3
#define SYSTEM_MONITOR_TEST 4

#define ADC_IS_BUSY 1
#define ADC_IS_FREE 0

//**************************************************
// Power Control PWM Module
//**************************************************

// e.g. at 40MHz clock and 40KHz PWM the resolution is
// 10 bits so 1023 is the max PWM. Half of it is 511

#define POWER_CONTROL_HALF_RESOLUTION 511

//**************************************************
// PWM available frequency
// this value points the index within the array
// on which LAP and Signed Magnitude PWM frequency are sored
//**************************************************

#define DEFAULT_PWM 30

#define PWM_FREQ_1KHZ 0
#define PWM_FREQ_2KHZ 1
#define PWM_FREQ_3KHZ 2
#define PWM_FREQ_4KHZ 3
#define PWM_FREQ_5KHZ 4
#define PWM_FREQ_6KHZ 5
#define PWM_FREQ_7KHZ 6
#define PWM_FREQ_8KHZ 7
#define PWM_FREQ_9KHZ 8
#define PWM_FREQ_10KHZ 9
#define PWM_FREQ_11KHZ 10
#define PWM_FREQ_12KHZ 11
#define PWM_FREQ_13KHZ 12
#define PWM_FREQ_14KHZ 13
#define PWM_FREQ_15KHZ 14
#define PWM_FREQ_16KHZ 15
#define PWM_FREQ_17KHZ 16
#define PWM_FREQ_18KHZ 17
#define PWM_FREQ_19KHZ 18
#define PWM_FREQ_20KHZ 19
#define PWM_FREQ_21KHZ 20
#define PWM_FREQ_22KHZ 21
#define PWM_FREQ_23KHZ 22
#define PWM_FREQ_24KHZ 23
#define PWM_FREQ_25KHZ 24
#define PWM_FREQ_26KHZ 25
#define PWM_FREQ_27KHZ 26
#define PWM_FREQ_28KHZ 27
#define PWM_FREQ_29KHZ 28
#define PWM_FREQ_30KHZ 29
#define PWM_FREQ_31KHZ 30
#define PWM_FREQ_32KHZ 31
#define PWM_FREQ_33KHZ 32
#define PWM_FREQ_34KHZ 33
#define PWM_FREQ_35KHZ 34
#define PWM_FREQ_36KHZ 35
#define PWM_FREQ_37KHZ 36
#define PWM_FREQ_38KHZ 37
#define PWM_FREQ_39KHZ 38
#define PWM_FREQ_40KHZ 39
#define PWM_FREQ_41KHZ 40
#define PWM_FREQ_42KHZ 41
#define PWM_FREQ_43KHZ 42
#define PWM_FREQ_44KHZ 43
#define PWM_FREQ_45KHZ 44
#define PWM_FREQ_46KHZ 45
#define PWM_FREQ_47KHZ 46
#define PWM_FREQ_48KHZ 47
#define PWM_FREQ_49KHZ 48
#define PWM_FREQ_50KHZ 49

//**************************************************
// PWM Frequency for LAP (standard CCP module is used)
//**************************************************

// The following values can be use if Prescaler is set to 16 T2_PS_1_16
#define PWM_SIGNED_MAGNITUDE_FREQ_1KHZ 25   // this value is an approssimation
#define PWM_SIGNED_MAGNITUDE_FREQ_2KHZ 255  // this value is an approssimation
#define PWM_SIGNED_MAGNITUDE_FREQ_3KHZ 207
#define PWM_SIGNED_MAGNITUDE_FREQ_4KHZ 155
#define PWM_SIGNED_MAGNITUDE_FREQ_5KHZ 124
#define PWM_SIGNED_MAGNITUDE_FREQ_6KHZ 103
#define PWM_SIGNED_MAGNITUDE_FREQ_7KHZ 88
#define PWM_SIGNED_MAGNITUDE_FREQ_8KHZ 77
#define PWM_SIGNED_MAGNITUDE_FREQ_9KHZ 68
#define PWM_SIGNED_MAGNITUDE_FREQ_10KHZ 62
#define PWM_SIGNED_MAGNITUDE_FREQ_11KHZ 56
#define PWM_SIGNED_MAGNITUDE_FREQ_12KHZ 51
#define PWM_SIGNED_MAGNITUDE_FREQ_13KHZ 47
#define PWM_SIGNED_MAGNITUDE_FREQ_14KHZ 44
#define PWM_SIGNED_MAGNITUDE_FREQ_15KHZ 41
#define PWM_SIGNED_MAGNITUDE_FREQ_16KHZ 38
#define PWM_SIGNED_MAGNITUDE_FREQ_17KHZ 36
#define PWM_SIGNED_MAGNITUDE_FREQ_18KHZ 34
#define PWM_SIGNED_MAGNITUDE_FREQ_19KHZ 32
#define PWM_SIGNED_MAGNITUDE_FREQ_20KHZ 30
#define PWM_SIGNED_MAGNITUDE_FREQ_21KHZ 29
#define PWM_SIGNED_MAGNITUDE_FREQ_22KHZ 27
#define PWM_SIGNED_MAGNITUDE_FREQ_23KHZ 26
#define PWM_SIGNED_MAGNITUDE_FREQ_24KHZ 25
#define PWM_SIGNED_MAGNITUDE_FREQ_25KHZ 24
#define PWM_SIGNED_MAGNITUDE_FREQ_26KHZ 23
#define PWM_SIGNED_MAGNITUDE_FREQ_27KHZ 22
#define PWM_SIGNED_MAGNITUDE_FREQ_28KHZ 21
#define PWM_SIGNED_MAGNITUDE_FREQ_29KHZ 21
#define PWM_SIGNED_MAGNITUDE_FREQ_30KHZ 20
#define PWM_SIGNED_MAGNITUDE_FREQ_31KHZ 19
#define PWM_SIGNED_MAGNITUDE_FREQ_32KHZ 19
#define PWM_SIGNED_MAGNITUDE_FREQ_33KHZ 18
#define PWM_SIGNED_MAGNITUDE_FREQ_34KHZ 17
#define PWM_SIGNED_MAGNITUDE_FREQ_35KHZ 17
#define PWM_SIGNED_MAGNITUDE_FREQ_36KHZ 16
#define PWM_SIGNED_MAGNITUDE_FREQ_37KHZ 16
#define PWM_SIGNED_MAGNITUDE_FREQ_38KHZ 15
#define PWM_SIGNED_MAGNITUDE_FREQ_39KHZ 15
#define PWM_SIGNED_MAGNITUDE_FREQ_40KHZ 15
#define PWM_SIGNED_MAGNITUDE_FREQ_41KHZ 14
#define PWM_SIGNED_MAGNITUDE_FREQ_42KHZ 14
#define PWM_SIGNED_MAGNITUDE_FREQ_43KHZ 14
#define PWM_SIGNED_MAGNITUDE_FREQ_44KHZ 13
#define PWM_SIGNED_MAGNITUDE_FREQ_45KHZ 13
#define PWM_SIGNED_MAGNITUDE_FREQ_46KHZ 13
#define PWM_SIGNED_MAGNITUDE_FREQ_47KHZ 12
#define PWM_SIGNED_MAGNITUDE_FREQ_48KHZ 12
#define PWM_SIGNED_MAGNITUDE_FREQ_49KHZ 12
#define PWM_SIGNED_MAGNITUDE_FREQ_50KHZ 12


// The following values can be use if Prescaler is set to 4 T2_PS_1_4
// Minimum Frequency ios set to 20KHz

//#define PWM_SIGNED_MAGNITUDE_FREQ_1KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_2KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_3KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_4KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_5KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_6KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_7KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_8KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_9KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_10KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_11KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_12KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_13KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_14KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_15KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_16KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_17KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_18KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_19KHZ 0
//#define PWM_SIGNED_MAGNITUDE_FREQ_20KHZ 120
//#define PWM_SIGNED_MAGNITUDE_FREQ_21KHZ 116
//#define PWM_SIGNED_MAGNITUDE_FREQ_22KHZ 108
//#define PWM_SIGNED_MAGNITUDE_FREQ_23KHZ 104
//#define PWM_SIGNED_MAGNITUDE_FREQ_24KHZ 100
//#define PWM_SIGNED_MAGNITUDE_FREQ_25KHZ 96
//#define PWM_SIGNED_MAGNITUDE_FREQ_26KHZ 92
//#define PWM_SIGNED_MAGNITUDE_FREQ_27KHZ 88
//#define PWM_SIGNED_MAGNITUDE_FREQ_28KHZ 84
//#define PWM_SIGNED_MAGNITUDE_FREQ_29KHZ 84
//#define PWM_SIGNED_MAGNITUDE_FREQ_30KHZ 80
//#define PWM_SIGNED_MAGNITUDE_FREQ_31KHZ 76
//#define PWM_SIGNED_MAGNITUDE_FREQ_32KHZ 76
//#define PWM_SIGNED_MAGNITUDE_FREQ_33KHZ 72
//#define PWM_SIGNED_MAGNITUDE_FREQ_34KHZ 68
//#define PWM_SIGNED_MAGNITUDE_FREQ_35KHZ 68
//#define PWM_SIGNED_MAGNITUDE_FREQ_36KHZ 64
//#define PWM_SIGNED_MAGNITUDE_FREQ_37KHZ 64
//#define PWM_SIGNED_MAGNITUDE_FREQ_38KHZ 60
//#define PWM_SIGNED_MAGNITUDE_FREQ_39KHZ 60
//#define PWM_SIGNED_MAGNITUDE_FREQ_40KHZ 60
//#define PWM_SIGNED_MAGNITUDE_FREQ_41KHZ 56
//#define PWM_SIGNED_MAGNITUDE_FREQ_42KHZ 56
//#define PWM_SIGNED_MAGNITUDE_FREQ_43KHZ 56
//#define PWM_SIGNED_MAGNITUDE_FREQ_44KHZ 52
//#define PWM_SIGNED_MAGNITUDE_FREQ_45KHZ 52
//#define PWM_SIGNED_MAGNITUDE_FREQ_46KHZ 52
//#define PWM_SIGNED_MAGNITUDE_FREQ_47KHZ 48
//#define PWM_SIGNED_MAGNITUDE_FREQ_48KHZ 48
//#define PWM_SIGNED_MAGNITUDE_FREQ_49KHZ 48
//#define PWM_SIGNED_MAGNITUDE_FREQ_50KHZ 48



/**
 * @note This table includes all the SIGNED MAGNITUDE PWM values to set the correct Frequency. 
 *       The array index is pointed by the PWM_FREQ_xxKHZ used when the motor initialization is called.
 */
const int pwm_SIGNED_MAGNITUDE_frequency_table [] = {PWM_SIGNED_MAGNITUDE_FREQ_1KHZ,PWM_SIGNED_MAGNITUDE_FREQ_2KHZ,PWM_SIGNED_MAGNITUDE_FREQ_3KHZ,
  									  			   PWM_SIGNED_MAGNITUDE_FREQ_4KHZ,PWM_SIGNED_MAGNITUDE_FREQ_5KHZ, PWM_SIGNED_MAGNITUDE_FREQ_6KHZ,
    							                   PWM_SIGNED_MAGNITUDE_FREQ_7KHZ,PWM_SIGNED_MAGNITUDE_FREQ_8KHZ,PWM_SIGNED_MAGNITUDE_FREQ_9KHZ,
								                   PWM_SIGNED_MAGNITUDE_FREQ_10KHZ,PWM_SIGNED_MAGNITUDE_FREQ_11KHZ,PWM_SIGNED_MAGNITUDE_FREQ_12KHZ,
									               PWM_SIGNED_MAGNITUDE_FREQ_13KHZ,PWM_SIGNED_MAGNITUDE_FREQ_14KHZ,PWM_SIGNED_MAGNITUDE_FREQ_15KHZ,
                                                   PWM_SIGNED_MAGNITUDE_FREQ_16KHZ,PWM_SIGNED_MAGNITUDE_FREQ_17KHZ,PWM_SIGNED_MAGNITUDE_FREQ_18KHZ,
								                   PWM_SIGNED_MAGNITUDE_FREQ_19KHZ,PWM_SIGNED_MAGNITUDE_FREQ_20KHZ,PWM_SIGNED_MAGNITUDE_FREQ_21KHZ,
								                   PWM_SIGNED_MAGNITUDE_FREQ_22KHZ,PWM_SIGNED_MAGNITUDE_FREQ_23KHZ,PWM_SIGNED_MAGNITUDE_FREQ_24KHZ,
								                   PWM_SIGNED_MAGNITUDE_FREQ_25KHZ,PWM_SIGNED_MAGNITUDE_FREQ_26KHZ,PWM_SIGNED_MAGNITUDE_FREQ_27KHZ,
  									               PWM_SIGNED_MAGNITUDE_FREQ_28KHZ,PWM_SIGNED_MAGNITUDE_FREQ_29KHZ,PWM_SIGNED_MAGNITUDE_FREQ_30KHZ,
    							                   PWM_SIGNED_MAGNITUDE_FREQ_31KHZ,PWM_SIGNED_MAGNITUDE_FREQ_32KHZ,PWM_SIGNED_MAGNITUDE_FREQ_33KHZ,
								                   PWM_SIGNED_MAGNITUDE_FREQ_34KHZ,PWM_SIGNED_MAGNITUDE_FREQ_35KHZ,PWM_SIGNED_MAGNITUDE_FREQ_36KHZ,
									               PWM_SIGNED_MAGNITUDE_FREQ_37KHZ,PWM_SIGNED_MAGNITUDE_FREQ_38KHZ,PWM_SIGNED_MAGNITUDE_FREQ_39KHZ,
                                                   PWM_SIGNED_MAGNITUDE_FREQ_40KHZ,PWM_SIGNED_MAGNITUDE_FREQ_41KHZ,PWM_SIGNED_MAGNITUDE_FREQ_42KHZ,
									               PWM_SIGNED_MAGNITUDE_FREQ_43KHZ,PWM_SIGNED_MAGNITUDE_FREQ_44KHZ,PWM_SIGNED_MAGNITUDE_FREQ_45KHZ,
									               PWM_SIGNED_MAGNITUDE_FREQ_46KHZ,PWM_SIGNED_MAGNITUDE_FREQ_47KHZ,PWM_SIGNED_MAGNITUDE_FREQ_48KHZ,
									               PWM_SIGNED_MAGNITUDE_FREQ_49KHZ,PWM_SIGNED_MAGNITUDE_FREQ_50KHZ 
								                 };
 
//**************************************************
// PWM Frequency for LAP (standard CCP module is used)
//**************************************************

#ifdef DC_MOTOR

    #define PWM_LAP_FREQ_1KHZ 2499
    #define PWM_LAP_FREQ_2KHZ 1249
    #define PWM_LAP_FREQ_3KHZ 832
    #define PWM_LAP_FREQ_4KHZ 624
    #define PWM_LAP_FREQ_5KHZ 499
    #define PWM_LAP_FREQ_6KHZ 416
    #define PWM_LAP_FREQ_7KHZ 356
    #define PWM_LAP_FREQ_8KHZ 312
    #define PWM_LAP_FREQ_9KHZ 277
    #define PWM_LAP_FREQ_10KHZ 249
    #define PWM_LAP_FREQ_11KHZ 226
    #define PWM_LAP_FREQ_12KHZ 207
    #define PWM_LAP_FREQ_13KHZ 191
    #define PWM_LAP_FREQ_14KHZ 178
    #define PWM_LAP_FREQ_15KHZ 166
    #define PWM_LAP_FREQ_16KHZ 155
    #define PWM_LAP_FREQ_17KHZ 146
    #define PWM_LAP_FREQ_18KHZ 138
    #define PWM_LAP_FREQ_19KHZ 131
    #define PWM_LAP_FREQ_20KHZ 124
    #define PWM_LAP_FREQ_21KHZ 118
    #define PWM_LAP_FREQ_22KHZ 113
    #define PWM_LAP_FREQ_23KHZ 108
    #define PWM_LAP_FREQ_24KHZ 103
    #define PWM_LAP_FREQ_25KHZ 99
    #define PWM_LAP_FREQ_26KHZ 95
    #define PWM_LAP_FREQ_27KHZ 92
    #define PWM_LAP_FREQ_28KHZ 88
    #define PWM_LAP_FREQ_29KHZ 85
    #define PWM_LAP_FREQ_30KHZ 82
    #define PWM_LAP_FREQ_31KHZ 80
    #define PWM_LAP_FREQ_32KHZ 77
    #define PWM_LAP_FREQ_33KHZ 75
    #define PWM_LAP_FREQ_34KHZ 73
    #define PWM_LAP_FREQ_35KHZ 70
    #define PWM_LAP_FREQ_36KHZ 68
    #define PWM_LAP_FREQ_37KHZ 67
    #define PWM_LAP_FREQ_38KHZ 65
    #define PWM_LAP_FREQ_39KHZ 63
    #define PWM_LAP_FREQ_40KHZ 62
    #define PWM_LAP_FREQ_41KHZ 60
    #define PWM_LAP_FREQ_42KHZ 59
    #define PWM_LAP_FREQ_43KHZ 57
    #define PWM_LAP_FREQ_44KHZ 56
    #define PWM_LAP_FREQ_45KHZ 55
    #define PWM_LAP_FREQ_46KHZ 53
    #define PWM_LAP_FREQ_47KHZ 52
    #define PWM_LAP_FREQ_48KHZ 51
    #define PWM_LAP_FREQ_49KHZ 50
    #define PWM_LAP_FREQ_50KHZ 49

#endif

/**
 * @note This table include all the LAP PWM values to set the correct Frequency. 
 *       The array index is pointed by the PWM_FREQ_xxKHZ used when the motor initialization is called.
 */

#ifdef DC_MOTOR
const int pwm_LAP_frequency_table [] = {PWM_LAP_FREQ_1KHZ,PWM_LAP_FREQ_2KHZ,PWM_LAP_FREQ_3KHZ,
  									  PWM_LAP_FREQ_4KHZ, PWM_LAP_FREQ_5KHZ, PWM_LAP_FREQ_6KHZ,
    							      PWM_LAP_FREQ_7KHZ,PWM_LAP_FREQ_8KHZ,PWM_LAP_FREQ_9KHZ,
								      PWM_LAP_FREQ_10KHZ,PWM_LAP_FREQ_11KHZ,PWM_LAP_FREQ_12KHZ,
									  PWM_LAP_FREQ_13KHZ,PWM_LAP_FREQ_14KHZ,PWM_LAP_FREQ_15KHZ,
                                      PWM_LAP_FREQ_16KHZ,PWM_LAP_FREQ_17KHZ,PWM_LAP_FREQ_18KHZ,
									  PWM_LAP_FREQ_19KHZ,PWM_LAP_FREQ_20KHZ,PWM_LAP_FREQ_21KHZ,
									  PWM_LAP_FREQ_22KHZ,PWM_LAP_FREQ_23KHZ,PWM_LAP_FREQ_24KHZ,
									  PWM_LAP_FREQ_25KHZ,PWM_LAP_FREQ_26KHZ,PWM_LAP_FREQ_27KHZ,
  									  PWM_LAP_FREQ_28KHZ,PWM_LAP_FREQ_29KHZ,PWM_LAP_FREQ_30KHZ,
    							      PWM_LAP_FREQ_31KHZ,PWM_LAP_FREQ_32KHZ,PWM_LAP_FREQ_33KHZ,
								      PWM_LAP_FREQ_34KHZ,PWM_LAP_FREQ_35KHZ,PWM_LAP_FREQ_36KHZ,
									  PWM_LAP_FREQ_37KHZ,PWM_LAP_FREQ_38KHZ,PWM_LAP_FREQ_39KHZ,
                                      PWM_LAP_FREQ_40KHZ,PWM_LAP_FREQ_41KHZ,PWM_LAP_FREQ_42KHZ,
									  PWM_LAP_FREQ_43KHZ,PWM_LAP_FREQ_44KHZ,PWM_LAP_FREQ_45KHZ,
									  PWM_LAP_FREQ_46KHZ,PWM_LAP_FREQ_47KHZ,PWM_LAP_FREQ_48KHZ,
									  PWM_LAP_FREQ_49KHZ,PWM_LAP_FREQ_50KHZ 
								      };
#endif

//**************************************************
// Irms Motor Current Table
//**************************************************

// The actual current is time 10. So 25 is equal to 250mA.
// System current is returning the current as the table below

#define CURRENT_0 0
#define CURRENT_1 25
#define CURRENT_2 50
#define CURRENT_3 75
#define CURRENT_4 100
#define CURRENT_5 125
#define CURRENT_6 150
#define CURRENT_7 175
#define CURRENT_8 200
#define CURRENT_9 225
#define CURRENT_10 250
#define CURRENT_11 275
#define CURRENT_12 300
#define CURRENT_13 325
#define CURRENT_14 350
#define CURRENT_15 375

const int I_peak_current_table [] = {CURRENT_0, CURRENT_1, CURRENT_2, CURRENT_3, CURRENT_4, CURRENT_5, CURRENT_6, CURRENT_7,
                                   CURRENT_8, CURRENT_9, CURRENT_10, CURRENT_11, CURRENT_12, CURRENT_13 ,CURRENT_14, CURRENT_15};

//**************************************************
// RS485 Baud Rate and interrupt
//**************************************************
// The vlaues defined here are related to the SPBRG register.
// Refer to the datasheet for more information and additional values.

#define BAUD_RATE_9600 		255
#define BAUD_RATE_19200 	129
#define BAUD_RATE_57600 	42
#define BAUD_RATE_115200 	21

#define TX_INTERRUPT_ON		USART_TX_INT_ON
#define TX_INTERRUPT_OFF	USART_TX_INT_OFF
#define RX_INTERRUPT_ON		USART_RX_INT_ON
#define RX_INTERRUPT_OFF	USART_RX_INT_OFF


//**************************************************
// Returned Values List
//**************************************************

#define EMG_STOP_NOT_ACTIVATED 2
#define OPERATION_PROPERLY_EXECUTED 1

#define ERROR_MODALITY_NOT_DEFINED -1
#define ERROR_RX_DATA_NOT_READY -2
#define ERROR_VIN_MAX -3
#define ERROR_VIN_MIN -4
#define ERROR_EMG_STOP_ACTIVE -5


//**************************************************
// Function Prototypes
//**************************************************

/**
 * The functions initializes the board and hardware.
 * All the PIC pins are properly set either as Input or Output. 
 * It should be called one time right after the main () function.
 *
 * @param 
 *
 * @return error: Refer to the defined error constants.
 */

signed char board_initialization (void);



/**
 * The function activates and initializes the UART used for the RS485 communication.
 * It should be called before using the RS485 bus and after having initialized the board.
 *
 * @param baud_rate  Is the communication baud rate. Value [BAUD_RATE_9600, BAUD_RATE_19200, BAUD_RATE_57600, BAUD_RATE_115200 ] 
 *
 * @param rx_interrupt_enable Enable or deactivate the RX interrupt. Value [RX_INTERRUPT_ON, RX_INTERRUPT_OFF]
 *
 * @param tx_interrupt_enable Enable or deactivate the RX interrupt. Calue [TX_INTERRUPT_ON, TX_INTERRUPT_OFF]
 *
 * @return error: Refer to the defined error constants.
 *
 * @note The UART settings, beside the baudrate that can be selected are: 8 bit format, 1 bit stop, No parity and No hardware handshaking.
 */

signed char initialize_RS485 (unsigned char baud_rate, unsigned char rx_interrupt_enable, unsigned char tx_interrupt_enable);



/**
 * The function reads the data out of the UART input buffer. The UART Input buffer is filled with the data coming from the RS485 bus.
 * The function is a non blocking function. 
 * If no data is within the buffer it returns ERROR_RX_DATA_NOT_READY, otherwise OPERATION_PROPERLY_EXECUTED.
 *
 * @param *data Pointer to the unsigned char that will contain the received data. 
 *
 * @return error: Refer to the defined error constants.
 */

signed char read_data_RS485 (unsigned char * data);



/**
 * The function writes a constant string stored to the rom area, to the UART.
 * The function is a blocking function until the data is sent out.
 * RS485 is automaticly changed to READ_WRITE mode and than back to READ_ONLY once the data is sent out.
 * 
 * @param *data Pointer to a constant string (eg. write_const_string_RS485 ("Hello World")).
 *
 * @return error: Refer to the defined error constants.
 */

signed char write_const_string_RS485 (const char *data);



/**
 * The function writes an Integer to the RS485 bus with a defined number of digits.
 * The function is a blocking function until the data is sent out.
 * RS485 is automaticly changed to READ_WRITE mode and than back to READ_ONLY once the data is sent out.
 * 
 * @param value Integer value to be sent on the RS485 bus.
 *
 * @number_of_digits Number of significant digits out of the integer number digits.
 *
 * @return error: Refer to the defined error constants.
 */

signed char write_int_ASCII_RS485 (int value, char number_of_digits);

/**
 * The function sends a Carriage Return via RS485.
 *
 * @param
 *
 * @return
 */

void send_return_RS485 (void);


/**
 * The function sends a Delete ASCII code via RS485.
 *
 * @param
 *
 * @return
 */
void send_delete_RS485 (void);


/**
 * The function reads the position of the dip switches.
 * Jumper JP1 must be properly set if dip switch 4 must be used.
 *
 * @param 
 *
 * @return dip_switch_value It returns the dip switch value.
 */

unsigned char get_dip_switch_value (void);


/**
 * The function initializes the DC motors either as LAP or Signed Magnitude modality.
 * The function should be called just once, before using the motor functions. 
 * It also sets the PWM at which the motors will be driven and the maximum current that will be monitored.
 * Maximum current is monitored using the check_system () function, that should be called periodicaly. 
 *
 * @param modality  It sets the motor modality eiter as LAP or Signed Magnitude. Value [LAP, SIGNED_MAGNITUDE]
 *
 * @param pwm_frequency It sets the PWM frequency. Value [PWM_FREQ_1KHZ - PWM_FREQ_50KHZ].
 *
 * @param max_current it sets the maximum pick current used by the motor expressed in mA. Value [min. 1, max. 5000]    
 *
 * @return error: Refer to the defined error constants.
 *
 * @note The PWM frequency is not optimyzed to let the user gets the maximum PWM resolution. 
 *       Timer Prescaler should be set differently if an optimiziation is wanted or required.
 */

#ifdef DC_MOTOR
signed char initialize_DC_motor (unsigned char modality, int pwm_frequency, int max_current );
#endif

/**
 * The functions rotates the DC motor clockwise.
 * The motors should be properly initialized before this function can be used.
 * 
 * @param motor Indicates the motor that should be rotated clockwise. Value [MOTOR_1, MOTOR_2, MOTOR_ALL]
 *
 * @param speed Motor speed. Value [ min. 1- max. 30000]
 *
 * @return error: Refer to the defined error constants.
 *
 * @warning Currently the speed is not monitored and the value does not acually represents the 
 *       rpm (root per minute)but the value written inside the PWM period register.
 *       For that reason care should be taken to not exceed the maximum PWM resolution.
 */
#ifdef DC_MOTOR
signed char rotate_motor_clockwise (unsigned char motor, int speed);
#endif


/**
 * the function rotates the DC motor anticlockwise.
 * The motors should be properly initialized before this function can be used.
 * 
 * @param motor Indicates the motor that should be rotated anticlockwise. Value [MOTOR_1, MOTOR_2, MOTOR_ALL]
 *
 * @param speed Motor speed. Value [ min. 1- max. 30000]
 *
 * @return error: Refer to the defined error constants.
 *
 * @warning Currently the speed is not monitored and the value does not acually represents the 
 *       rpm (root per minute)but the value written inside the PWM period register.
 *       For that reason care should be taken to not exceed the maximum PWM resolution.
 */
#ifdef DC_MOTOR
signed char rotate_motor_anticlockwise (unsigned char motor, int speed);
#endif


/**
 * The function stops the motor rotation, either DC motor or stepper motor.
 * The motors should be properly initialized before this function can be used.
 * It could be used for an emergency stop, beside other actions.
 * 
 * @param motor Indicates the motor that should be halted. Value [MOTOR_1, MOTOR_2, MOTOR_ALL]
 *
 * @return error: Refer to the defined error constants.
 *
 * @note If a Stepper motor is used the input parameter is ignored [MOTOR_1, MOTOR_2, MOTOR_ALL], so any choise would be fine..
 *       For code consistency MOTOR_1 is reccomended to be used. 
 *
 */

signed char stop_motor_rotation (unsigned char motor);



/**
 * The function initializes the stepper motor either in Full step, full step two phases or half step.
 * The funtion should be called once, before using the stepper motor functions. 
 * It also sets the PWM at which the motor will be driven and the maximum current that will be monitored.
 * Maximum current is monitored using the check_system () function, that should be called periodicaly. 
 *
 * @param modality  It sets the motor modality in Full step, full step two phases or half step. Value [FULL_STEP, FULL_STEP_2_PHASES, HALF_STEP)
 *
 * @param pwm_frequency It sest the PWM frequency. Value [PWM_FREQ_1KHZ - PWM_FREQ_50KHZ].
 *
 * @param max_current It sets the maximum pick current used by the motor expressed in mA. Value [min. 0, max. 5000]    
 *
 * @return error: Refer to the defined error constants.
 *
 * @note The PWM frequency is not optimyzed to let the user gets the maximum PWM resolution. 
 *       Timer Prescaler should be set differently if an optimiziation is wanted or required.
 */

signed char initialize_stepper_motor (unsigned char modality, unsigned char pwm_frequency, int max_current );



/**
 * The function rotates the stepper motor clockwise at certain torque.
 * The motors should be properly initialized before this function can be used.
 * 
 * @param current Phase current wanted for that rotation expressed in mA. Value [ min. 1 - max. 4000]
 *
 * @return error: Refer to the defined error constants.
 *
 * @warning Currently the current is not monitored and the value does not acually represents the 
 *       wanted current but the value written inside the PWM period register. 
 *       For that reason care should be taken to not excede the maximum PWM resolution.
 */

signed char step_motor_clockwise (int current);


/**
 * The function rotates the stepper motor anticlockwise at certain torque.
 * The motors should be properly initialized before this function can be used.
 * 
 * @param current Phase current wanted for that rotation expressed in mA. Value [ min. 1 - max. 4000]
 *
 * @return error: refer to the defined error constants
 *
 * @warning Currently the current is not monitored and the value does not acually represents the 
 *       wanted current but the value written inside the PWM period register. 
 *       For that reason care should be taken to not excede the maximum PWM resolution.
 */
signed char step_motor_anticlockwise (int current);


/**
 * The function reads the the voltage from the potenziometer POT_1 connected to AUX_1 connector.
 * 
 * @param 
 *
 * @return Analog value from the Potenziometer POT_1 connected to AUX_1. The value is expressed in quantum. 
 *
 * @note The ADC reading is a blocking function until the ADC conversion is over and the ADC buffer is read out.
 *
 */

int ADC_read_POT_1 (void );


/**
 * The function reads the the voltage from the potenziometer POT_2 connected to AUX_2 connector.
 * 
 * @param 
 *
 * @return Analog value from the Potenziometer POT_2 connected to AUX_2. The value is expressed in quantum.  
 *
 * @note The ADC reading is a blocking function until the ADC conversion is over and the ADC buffer is read out.
 *
 */

int ADC_read_POT_2 (void );



/**
 * The function reads the the voltage from the temperature sensor AN_TEMP and returns the reading expressed in celsius degree.
 * This function is called from system_check () to monitor that the temperature is less than TEMP_MAX. If the themperature 
 * reachs TEMP_WARNING the fun is turned on.
 * 
 * @param 
 *
 * @return Analog value from the temperature sensor AN_TEMP expressed in celsius degree.
 *
 * @note The ADC reading is a blocking function until the ADC conversion is over and the ADC buffer is read out.
 *
 */

int ADC_read_TEMP (void );



/**
 * The function reads the the voltage from the temperature sensor VIN_TEST and returns the reading expressed in V.
 * This function is called from system_check () to monitor that the voltage is within VIN_MIN and VIN_MAX.
 * 
 * @param 
 *
 * @return Analog value from the temperature sensor VIN_TEST expressed in V.
 *
 * @note The ADC reading is a blocking function until the ADC conversion is over and the ADC buffer is read out.
 *
 */

int ADC_read_VIN_TEST (void );


/**
 * The function reads the the current sinked from the selected Motor. The returned value is expressed in mA.
 * This function is called from system_check () to monitor that the curernt is within maximum pick motor current.
 * 
 * @param motor Select the motor from which the curren shlould be read. Value [MOTOR_1, MOTOR_2].
 *
 * @return Current expressed in mA from the selected motor.
 *
 * @note The ADC reading is a blocking function until the ADC conversion is over and the ADC buffer is read out.
 *
 */

int ADC_read_motor_current (unsigned char motor );



/**
 * The function turns the Fun ON. If system_check is regularly called, the fun is turned on if the themperature gets higher than TEMP_WARNING.
 * 
 * @param
 *
 * @return
 *
 */

void turn_on_FAN (void);



/**
 * The function turns the Fun OFF. If system_check is regularly called, the fun is turned off if the themperature gets lower than TEMP_WARNING - TEMP_HYSTERESIS.
 * 
 * @param
 *
 * @return
 *
 */
void turn_off_FAN (void);



/**
 * The function checks that the system is working within the maximum values. 
 * Temperature, Current and supply voltage are checked out. 
 * 
 * @param
 *
 * @return Status: system status 
 *
 * @note If an anomaly is detected the set_emergency_stop_out is called and the system is halted.
 *       ERROR_LED is turned on.
 * @note If themperature gets hiher than TEMP_MAX the motors are halted. If Themperature gets higher than TEMP_WARNING, the fun is turned ON.
 *       The fun is turned OFF, only if the Themperature gets reduced to TEMP_WARNING - TEMP_HYSTERESIS.
 *
 * @warning This function should be called periodically. If not periodically called the system is not monitored. 
 *
 */

signed char system_check (void);



/**
 * The function checks if the emergency stop OUT is activated. 
 * This is done monitoring the global flag associated to the emergency stop OUT. 
 * 
 * @param
 *
 * @return Status: Emergency stop status. Value [ACTIVATED, DEACTIVATED]
 *
 * @note Status 1: EMG_STOP_OUT is Active, 0: EMG_STOP_OUT is not Active
 *          
 *
 */

unsigned char is_emergency_stop_out_active (void);



/**
 * The function checks if the emergency stop IN is activated. 
 * This is done monitoring the global flag associated to the emergency stop IN. 
 * 
 * @param
 *
 * @return Status: Emergency stop status. Value [ACTIVATED, DEACTIVATED]
 *
 * @note Status 1: EMG_STOP_IN is Active, 0: EMG_STOP_IN is not Active
 *          
 */

unsigned char is_emergency_stop_in_active (void);



/**
 * The function sets the emergency stop OUT to ACTIVATED and the EMG_OUT pin is set to 1. 
 * The global flag associated to the emergency stop OUT is also set to ACTIVATED. 
 * 
 * @param
 *
 * @return Status: OPERATION_PROPERLY_EXECUTED
 *
 * @note Status 1: EMG_STOP_OUT is Active, 0: EMG_STOP_OUT is not Active
 *          
 */

unsigned char set_emergency_stop_out (void);



/**
 * The function sets the emergency stop IN to activated. 
 * The global flag associated to the emergency stop IN is also set to ACTIVATED. 
 * 
 * @param
 *
 * @return Status: OPERATION_PROPERLY_EXECUTED
 *
 * @note Status 1: EMG_STOP_IN is Active, 0: EMG_STOP_IN is not Active
 *          
 */

unsigned char set_emergency_stop_in (void);


/**
 * The function resets the emergency stop OUT and pin.
 * The global flag associated to the emergency stop OUT is also set to DEACTIVATED. 
 * 
 * @param
 *
 * @return Status: OPERATION_PROPERLY_EXECUTED
 *
 * @note Status 1: EMG_STOP_OUT is Active, 0: EMG_STOP_OUT is not Active
 *          
 */
unsigned char reset_emergency_stop_out (void);



/**
 * The function resets the emergency stop IN and pin.
 * The global flag associated to the emergency stop IN is also set to DEACTIVATED. 
 * 
 * @param
 *
 * @return Status: OPERATION_PROPERLY_EXECUTED
 *
 * @note Status 1: EMG_STOP_IN is Active, 0: EMG_STOP_IN is not Active
 *          
 */
unsigned char reset_emergency_stop_in (void);


/**
 * The function checks the emergency stop conditions. If either the emergency stop in or out are activeted
 * any motion activity is halted.
 * 
 * @param
 *
 * @return Status: EMG_STOP_NOT_ACTIVATED, ERROR_EMG_STOP_ACTIVE
 *
 *          
 */

unsigned char emergency_stops_check (void);


/******************************************************************************************************
//                                        PRIVATE FUNCTIONS
//
// The following functions should be considered PRIVATE, the are used by internal functions.
// Do not use or change it!
/******************************************************************************************************


/**
 * The function reads the ADC buffer after ADC conversion is completed. 
 * 
 * @param
 *
 * @return Status: It returns the ADC value out of the buffer.
 *
@warning This function should be consdired Private so it should not be used within the user applications.
 *          
 */
int ADC_read_buffer (void);



/**
 * The function sets the period of the Power Control PWM module (12 bit resolution).
 * 
 * @param period It sets the period of the Power Control PWM module (12 bit resolution). Value [ min. 0 - max. 4095].
 *        
 *
 * @return
 *
 * @warning The actual resolution depends on the Frequency and Prescaler settings.
 *
 * @warning This function should be consdired Private so it should not be used within the user applications.
 *          
 */

void power_control_PWM_set_period (int period);



/**
 * The function sets the period of the PWM 0 module (10 bit resolution).
 * 
 * @param duty_cycle It sets the period of the PWM 0 module (10 bit resolution). Value [ min. 0 - max. 1023].
 *        
 * @return
 *
 * @warning The actual resolution depends on the Frequency and Prescaler settings.
 *
 * @warning This function should be consdired Private so it should not be used within the user applications.
 *          
 */

void set_duty_cycle_DC0 (int duty_cycle);


/**
 * The function sets the period of the PWM 1 module (10 bit resolution).
 * 
 * @param duty_cycle It sets the period of the PWM 1 module (10 bit resolution). Value [ min. 0 - max. 1023].
 *        
 * @return
 *
 * @warning The actual resolution depends on the Frequency and Prescaler settings.
 *
 * @warning This function should be consdired Private so it should not be used within the user applications.
 *          
 */
void set_duty_cycle_DC1 (int duty_cycle); 



/**
 * The function allows enabling and disabling system interrupt.
 *
 * @param interrupt_status  It specifies whether the interrupts need to be activated or not.
 *
 * @return
 *
 *
 */
void initialize_interrupt (unsigned char interrupt_status);

/**
 * The function initializes the TIMER0 and activate the interuppt.
 * 
 * @param interrupt_priority  It specifies the Interrupt Priority Level
 *        
 * @return
 *
 *          
 */
void initialize_TIMER0 (unsigned char interrupt_priority);


/**
 * The function activate the interrupt for one of the follwowing signal ENABLE, 
 * STEP, DIRECTION, SWITCH
 *
 * @param interrupt_priority  It specifies the Interrupt Priority Level
 *
 * @return
 *
 *
 */
void activate_signal_interupt (unsigned char signal, unsigned char interrupt_priority );


/**
 * This function sest the error statud global variable accordingly to the Error flags.
 *
 * @param error  It specifies the error to be added to the status
 *
 * @return
 *
 */
void set_error_status (unsigned char error);


/**
 * This function returns the error status global variable.
 *
 * @param
 *
 * @return Error Status
 *
 */
unsigned char get_error_status (void);


/**
 * This function sets the warning status global variable accordingly to the Warning flags.
 *
 * @param warning  It specifies the warning to be added to the status
 *
 * @return
 *
 */
void set_warning_status (unsigned char warning);

/**
 * This function returns the warning status global variable.
 *
 * @param
 *
 * @return Error Status
 *
 */
unsigned char get_warning_status (void);



#endif
